/*****************************************************************************/
/*!	\file 		QtClientWidget.h
 *	\brief          QtClient GUI header
 *	\par 		SystemCORP Embedded Technology Pty Ltd
 *
 *			5/50 William St.,
 *			Beckenham,
 *			Perth, WA 6107,
 *			Australia.
 *
 *			Phone	: +61 (8) 9258 4683
 *			Fax	: +61 (8) 9258 4673
 *			Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

#ifndef QTCLIENTWIDGET_H
#define QTCLIENTWIDGET_H
#include <QtGui>
#include <QWidget>
#include <QtGui/QApplication>
#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QLabel>
#include <QTimeLine>
#include <qpixmap.h>

#include "ui_QtClientWidget.h"

#include "IEC61850Types.h"
#include "IEC61850API.h"
#include "sysctype.h"


QT_BEGIN_NAMESPACE
class QLabel;
class QPixmap;
class QPropertyAnimation;
QT_END_NAMESPACE

namespace Ui {
    class QtClientWidget;
}

class QtClientWidget : public QWidget, private Ui::QtClientWidgetDLG
{
    Q_OBJECT

public:
    explicit QtClientWidget(QWidget *parent = 0);
    ~QtClientWidget();
    void *ptrToPIS10;

private:
   // Ui::QtServerWidget *ui;
    QLCDNumber *LcdNum;

public slots:
    void pshbtnCloseHandler();
    void pbAlrmHandler();
    void UpdateStateTimerHandler();
    void pshbtnReadAnIn1Handler();
    void pshbtnReadAnIn2Handler();
    void LedController(unsigned int uiLedIndx, Boolean bValue);
    void pbXcbrHandler();
    void pbCswiHandler();
    void pbDipSw1Handler();
    void pbDipSw2Handler();
    void pbDipSw3Handler();
    void pbDipSw4Handler();
    void AlarmBtnColorChange();
    void XCBR_CSWIController(unsigned int uiIndx, unsigned char cValue);

public:    
    QGraphicsScene *scene;
    QTimeLine *timeLine;
    QPixmap *pixmapGreenLed;
    QPixmap *pixmapRedLed;
    QPixmap *pixmapGreyLed;
    QPixmap *pixmapOrangeLed;
    QIcon *IconGreenLed;
    QIcon *IconRedLed;
    QTimer *timerUpdateState;
    QTimer *timerAlarmColor;
};

#endif // QTCLIENTWIDGET_H
