/*****************************************************************************/
/*!	\file 		pis10Client.h
 *	\brief          Wrapper header for PIS10 stack
 *	\par 		SystemCORP Embedded Technology Pty Ltd
 *
 *			5/50 William St.,
 *			Beckenham,
 *			Perth, WA 6107,
 *			Australia.
 *
 *			Phone	: +61 (8) 9258 4683
 *			Fax	: +61 (8) 9258 4673
 *			Email	: info@systemcorp.com.au
 */
/*****************************************************************************/
#ifndef PIS10CLIENT_H
#define PIS10CLIENT_H

#include "IEC61850Types.h"
#include "IEC61850API.h"
#include "sysctype.h"

#include "TestAppTypes.h"

#include <QObject>
#include "QtClientWidget.h"

class PIS10Client : public QObject
{
    Q_OBJECT
public: 
    static QtClientWidget * MyQTwidget;
    explicit PIS10Client(QObject *parent = 0);
    IEC61850 MyClient;
    static PIS10Client * pThis;

public slots:
    enum IEC61850_ErrorCodes CreateClient(unsigned long int inOptions);
    enum IEC61850_ErrorCodes LoadSCLFile(char *pFileName);
    enum IEC61850_ErrorCodes StartMyClient();
    enum IEC61850_ErrorCodes StopMyClient();
    enum IEC61850_ErrorCodes FreeMyClient();
    IEC61850 GetMyClient();
    enum IEC61850_ErrorCodes RunClient();
    int RunGUI(int argc, char *argv[]);

    //If this fails try to extern C these
    static int ReadCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData *ptReturnedValue);
    static int  WriteCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData * ptNewValue);
    static int  SelectCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptSelectValue, int iSyncroCheck, int iInterlockCheck);
    static int  OperateCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptOperateValue, int iSyncroCheck, int iInterlockCheck);
    static int  CancelCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID);
    static void  UpdateCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, const struct IEC61850_DataAttributeData *ptNewValue);

    /*
    static void SetReadThreadVars(struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData *ptReturnedValue);
    static void ThreadRead();
    */

};



#endif // PIS10CLIENT_H
