/*****************************************************************************/
/*!	\file 			QtServerWidget.cpp
 *	\brief                  Qt Server Widget
 *	\par 			SystemCORP Embedded Technology Pty Ltd
 *
 *				15/50 William St.,
 *				Beckenham,
 *				Perth, WA 6107,
 *				Australia.
 *
 *				Phone	: +61 (8) 9258 4683
 *				Fax	: +61 (8) 9258 4673
 *				Email	: info@systemcorp.com.au
 */
/*****************************************************************************/
/******************************************************************************
*	Include
******************************************************************************/
#include <QtGui>
#include <QtGui/QApplication>1
#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QLabel>
#include <qpixmap.h>
#include <QTimeLine>
#include <time.h>
#include "QtServerWidget.h"
#include "ui_QtServerWidget.h"
#include "pis10Server.h"

/******************************************************************************
* Status Variables
******************************************************************************/
#define MaxCount 999
#define IEC61850_CLIENT_INDEX 0

unsigned int phsABIn;
unsigned int phsAIn;
int imaxAlarmThreshold;
Boolean iAlarmFlg;
static char iToggle=0;

// DipSW Status flags
Boolean bDipSw1 =FALSE;
Boolean bDipSw2 =FALSE;
Boolean bDipSw3 =FALSE;
Boolean bDipSw4 =FALSE;
// Led Status flags
Boolean bLed1 =FALSE;
Boolean bLed2 =FALSE;
Boolean bLed3 =FALSE;
Boolean bLed4 =FALSE;
// XCBR CSWI Status flags
Boolean bErthSwFlg = 0;
Boolean bXCBRSwFlg = 0;

/*****************************************************************************
* Constructor
******************************************************************************
* Constructor for QtServerWidget.
*
******************************************************************************/
QtServerWidget::QtServerWidget(QWidget *parent) : QWidget(parent) //ui(new Ui::QtServerWidget)
{
        setupUi(this);
        setWindowFlags(Qt::FramelessWindowHint);
        QImage imgLogo("TQ Demo Banner.png");

        scene = new QGraphicsScene;
        QPixmap pixmap("switch00.png");
        scene->addPixmap(pixmap);

        // Load the LED image files
        pixmapGreenLed = new QPixmap("square_green.svg");
        pixmapRedLed = new QPixmap("square_red.svg");
        pixmapGreyLed = new QPixmap("square_grey.svg");
        pixmapOrangeLed = new QPixmap("square_orange.svg");

        IconGreenLed = new QIcon("square_green.svg");
        IconRedLed = new QIcon("square_red.svg");

        QFont* font = new QFont();
        font->setBold(true);
        font->setPixelSize(20);

        QPixmap pixmapESWOpenXCBROpen("switch00.png");
        QPixmap pixmapESWOpenXCBRClose("switch01.png");
        QPixmap pixmapESWCloseXCBROpen("switch10.png");
        QPixmap pixmapESWCloseXCBRClose("switch11.png");

        QPalette palspin = spinBox_AnIn1->palette();
        spinBox_AnIn1->setStyleSheet(
        "QSpinBox { border: 1px solid gray; border-radius: 5px;}"
        "QSpinBox::up-arrow { border-left: 17px solid none;" "border-right: 17px solid none; border-bottom: 17px solid black; width: 0px; height: 0px; }"
        "QSpinBox::up-arrow:hover { border-left: 17px solid none;""border-right: 17px solid none; border-bottom: 17px solid black; width: 0px; height: 0px; }"
        "QSpinBox::up-button { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.midlight().color())+") }"
        "QSpinBox::up-button:hover { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.shadow().color())+") }"

        "QSpinBox::down-arrow { border-left: 17px solid none;""border-right: 17px solid none; border-top: 17px solid black; width: 0px; height: 0px; }"
        "QSpinBox::down-arrow:hover { border-left: 17px solid none;""border-right: 17px solid none; border-top: 17px solid black; width: 0px; height: 0px; }"
        "QSpinBox::down-button { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.midlight().color())+") }"
        "QSpinBox::down-button:hover { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.shadow().color())+") }"
       );
       spinBox_AnIn1 ->setFont(*font);

       spinBox_AnIn2->setStyleSheet(
       "QSpinBox { border: 1px solid gray; border-radius: 5px;}"
       "QSpinBox::up-arrow { border-left: 17px solid none;""border-right: 17px solid none; border-bottom: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::up-arrow:hover { border-left: 17px solid none;" "border-right: 17px solid none; border-bottom: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::up-button { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.midlight().color())+") }"
       "QSpinBox::up-button:hover { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.shadow().color())+") }"

       "QSpinBox::down-arrow { border-left: 17px solid none;""border-right: 17px solid none; border-top: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::down-arrow:hover { border-left: 17px solid none;""border-right: 17px solid none; border-top: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::down-button { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.midlight().color())+") }"
       "QSpinBox::down-button:hover { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.shadow().color())+") }"
       );

       spinBox_AlmThrsld->setStyleSheet(
       "QSpinBox { border: 1px solid gray; border-radius: 5px;}"
       "QSpinBox::up-arrow { border-left: 17px solid none;""border-right: 17px solid none; border-bottom: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::up-arrow:hover { border-left: 17px solid none;""border-right: 17px solid none; border-bottom: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::up-button { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.midlight().color())+") }"
       "QSpinBox::up-button:hover { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.shadow().color())+") }"

       "QSpinBox::down-arrow { border-left: 17px solid none;""border-right: 17px solid none; border-top: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::down-arrow:hover { border-left: 17px solid none;""border-right: 17px solid none; border-top: 17px solid black; width: 0px; height: 0px; }"
       "QSpinBox::down-button { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.midlight().color())+") }"
       "QSpinBox::down-button:hover { width: 40px; height: 37px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0.25 "+getRGBhexColor(palspin.light().color())+", stop: 1 "+getRGBhexColor(palspin.shadow().color())+") }"
       );

        //ui->
        gvSw->setScene(scene);
        //ui->
        gvSw->show();
        //ui->
        lblLogo->setPixmap(QPixmap::fromImage(imgLogo));

        // Creating Timer for constant updates
        timerUpdateState = new QTimer(this);
        timerAlarmColor = new QTimer(this);

        // Set Default values
        bErthSwFlg = 0;
        bXCBRSwFlg = 0;
        imaxAlarmThreshold = 0;

        connect(timerAlarmColor, SIGNAL(timeout()), this, SLOT(AlarmBtnColorChange()));
        //Timers to handle the flashing of the buttons after an Operate has been performed.
        connect(timerUpdateState, SIGNAL(timeout()), this, SLOT(UpdateStateTimerHandler()));
        timerUpdateState->start(500);
}

/*****************************************************************************
* Destructor
******************************************************************************
* Destructor for for QtServerWidget.
*
******************************************************************************/
QtServerWidget::~QtServerWidget()
{
    //delete ui;
}

// Handler functions
/*****************************************************************************
* pbDipSw1Handler() Function
******************************************************************************
* This function is handler for DipSw1 it gets called when DipSw1 is clicked
*
******************************************************************************/
void QtServerWidget ::pbDipSw1Handler()
{
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bDipSw1)*8;
    tDAData.pvData = &bDipSw1;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 1;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    if(bDipSw1 == TRUE)
    {
        bDipSw1 = FALSE;
        pbDipSw1->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw1 = TRUE;
        pbDipSw1->setIcon(*IconGreenLed);
    }

    // Update the value in the stack
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Update value is %d \n", bDipSw1);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pbDipSw2Handler() Function
******************************************************************************
* This function is handler for DipSw2 it gets called when DipSw2 is clicked
*
******************************************************************************/
void QtServerWidget ::pbDipSw2Handler()
{
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bDipSw1)*8;
    tDAData.pvData = &bDipSw2;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 2;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    if(bDipSw2 == TRUE)
    {
        bDipSw2 = FALSE;
        pbDipSw2->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw2 = TRUE;
        pbDipSw2->setIcon(*IconGreenLed);
    }

    // Update the value in the stack
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Write value is %d \n", bDipSw2);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pbDipSw3Handler() Function
******************************************************************************
* This function is handler for DipSw3 it gets called when DipSw3 is clicked
*
******************************************************************************/
void QtServerWidget ::pbDipSw3Handler()
{
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bDipSw1)*8;
    tDAData.pvData = &bDipSw3;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 3;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    if(bDipSw3 == TRUE)
    {
        bDipSw3 = FALSE;
        pbDipSw3->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw3 = TRUE;
        pbDipSw3->setIcon(*IconGreenLed);
    }

     // Update the value in the stack
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Write value is %d \n", bDipSw3);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pbDipSw4Handler() Function
******************************************************************************
* This function is handler for DipSw4 it gets called when DipSw4 is clicked
*
******************************************************************************/
void QtServerWidget ::pbDipSw4Handler()
{
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bDipSw1)*8;
    tDAData.pvData = &bDipSw4;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 4;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    if(bDipSw4 == TRUE)
    {
        bDipSw4 = FALSE;
        pbDipSw4->setIcon(*IconRedLed);
    }
    else
    {
        bDipSw4 = TRUE;
        pbDipSw4->setIcon(*IconGreenLed);
    }
    // Update the value in the stack
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("Write value is %d \n", bDipSw4);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
}

/*****************************************************************************
* pshbtnWriteAnIn1Handler() Function
******************************************************************************
* This function is handler for AnIn1 it gets called when AnIn1 is clicked to
* set a value
******************************************************************************/
void QtServerWidget ::pshbtnUpdateAnIn1Handler()
{
    int iAnIn1;
    unsigned int iAlmThrsld;
    Boolean bAlmFlg = FALSE;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_INT32U;
    tDAData.uiBitLength = sizeof(iAnIn1)*8;
    tDAData.pvData = &iAnIn1;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 3;
    tDAID.uiField2 = 1;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    iAnIn1 = spinBox_AnIn1->value();
    iAlmThrsld = spinBox_AlmThrsld->value();

    phsABIn = iAnIn1;
    // Update  Analog  value in the stack
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("pshbtnWriteAnIn1Handler iAnIn1 Update value is %d \n", iAnIn1);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bAlmFlg)*8;
    tDAData.pvData = &bAlmFlg;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 5;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    // Update  Alarm  value in the stack
    if(phsABIn > iAlmThrsld)
    {
        bAlmFlg = TRUE;

        eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf("pshbtnWriteAnIn1Handler  bAlmFlg Update value is %d \n", bAlmFlg);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }
    else
    {
        bAlmFlg = FALSE;

        eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf(" pshbtnWriteAnIn1Handler bAlmFlg Update value is %d \n", bAlmFlg);
            //lcdNmSimVal->display(iValue);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }

    iAlarmFlg = bAlmFlg;
}

/*****************************************************************************
* pshbtnWriteAnIn2Handler() Function
******************************************************************************
* This function is handler for AnIn2 it gets called when AnIn2 is clicked to
* set a value
******************************************************************************/
void QtServerWidget ::pshbtnUpdateAnIn2Handler()
{
    unsigned int iAnIn2;
    unsigned int iAlmThrsld;
    Boolean bAlmFlg = FALSE;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_INT32U;
    tDAData.uiBitLength = sizeof(iAnIn2)*8;
    tDAData.pvData = &iAnIn2;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 3;
    tDAID.uiField2 = 2;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    iAnIn2 = spinBox_AnIn2->value();
    iAlmThrsld = spinBox_AlmThrsld->value();

    phsAIn = iAnIn2;
    // Update  Analog  value in the stack
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("pshbtnWriteAnIn2Handler iAnIn2 Update value is %d \n", iAnIn2);
    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bAlmFlg)*8;
    tDAData.pvData = &bAlmFlg;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 5;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;
    // Update  Alarm  value in the stack
    if(phsAIn > iAlmThrsld)
    {
        bAlmFlg = TRUE;

        eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf("pshbtnWriteAnIn2Handler bAlmFlg Update value is %d \n", bAlmFlg);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }
    else
    {
        bAlmFlg = FALSE;

        eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf(" pshbtnWriteAnIn2Handler bAlmFlgUpdate value is %d \n", iAnIn2);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }

    iAlarmFlg = bAlmFlg;
}

/*****************************************************************************
* MaxAlarmHandler() Function
******************************************************************************
* This function is handler for AlramThreshold it gets called when AlramThreshold
* is clicked to set a value
******************************************************************************/
void QtServerWidget :: MaxAlarmHandler()
{
    unsigned int iAlmThrsld;
    Boolean bAlmFlg = FALSE;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    iAlmThrsld = spinBox_AlmThrsld->value();

    tDAData.ucType = IEC61850_DATATYPE_INT32U;
    tDAData.uiBitLength = sizeof(iAlmThrsld)*8;
    tDAData.pvData = &iAlmThrsld;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 6;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;

    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf(" MaxAlarmHandler Update value is %d \n", iAlmThrsld);

    }
    else
    {
        printf("Got error on write %d \n", eError);
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);

    }

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bAlmFlg)*8;
    tDAData.pvData = &bAlmFlg;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;
    tDAID.uiField1 = 1;
    tDAID.uiField2 = 5;
    tDAID.uiField3 = 1;
    tDAID.uiField4 = 0;
    tDAID.uiField5 = 0;
    // Update  Alarm  value in the stack
    if(phsABIn > iAlmThrsld || phsAIn > iAlmThrsld)
    {
        bAlmFlg = TRUE;

        eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf(" MaxAlarmHandler Update value is %d \n", iAlmThrsld);

        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }
    else
    {
        bAlmFlg = FALSE;

        eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
        if(eError == IEC61850_ERROR_NONE)
        {
            printf(" MaxAlarmHandler Update value is %d \n", iAlmThrsld);
        }
        else
        {
            printf("Got error on write %d \n", eError);
            printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
        }
    }

    iAlarmFlg = bAlmFlg;

}

/*****************************************************************************
* LedController() Function
******************************************************************************
* This function is handler for Leds when client performs operate on LEDs
* IEC61850_OperateCallback function calls this function to set the flags
******************************************************************************/
void QtServerWidget ::LedController(unsigned int uiLedIndx, Boolean bValue)
{
    Boolean bLedStatus = FALSE;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(bLedStatus)*8;
    tDAData.pvData = &bLedStatus;

    tDAID.Generic_type = IEC61850_DAID_GENERIC;

    switch(uiLedIndx)
    {
        case 1:
        {
            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 2;
            tDAID.uiField2 = 1;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;

            if(bValue == TRUE)
            {
                bLed1 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed1 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 2:
        {
            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 2;
            tDAID.uiField2 = 2;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;
            if(bValue == TRUE)
            {
                bLed2 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed2 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 3:
        {
            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 2;
            tDAID.uiField2 = 3;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;

            if(bValue == TRUE)
            {
                bLed3 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed3 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        case 4:
        {
            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 2;
            tDAID.uiField2 = 4;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;
            if(bValue == TRUE)
            {
                bLed4 = TRUE;
                bLedStatus = TRUE;
            }
            else
            {
                bLed4 = FALSE;
                bLedStatus = FALSE;
            }
        }
        break;
        }

    //printf("LED uiIndx : %d,  bLedStatus: %d\n", uiLedIndx, bLedStatus);
    //Update LED status value
    eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
    }
    else
    {
        printf("Got error on update %d \n", eError);
    }
}

/*****************************************************************************
* XcbrCswController() Function
******************************************************************************
* This function is handler for XCBR & CSW Sw, when client performs operate on XCBR & CSW
* IEC61850_OperateCallback function calls this function to set the flags
******************************************************************************/
void QtServerWidget ::XcbrCswController(unsigned int uiIndx, unsigned char cValue)
{
    unsigned char bValue = IEC61850_DB_POS_FALSE;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;
    struct IEC61850_DataAttributeID_Generic tDAID; //
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_INT8U;
    tDAData.uiBitLength = sizeof(bValue)*8;
    tDAData.pvData = &bValue;

    //printf("uiIndx : %d,  cValue: %d\n", uiIndx, cValue);

    if(uiIndx == 4)
    {
        if(cValue == TRUE)
        {
            bValue = IEC61850_DB_POS_TRUE;
            printf("bValue: %d\n", bValue);

            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 4;
            tDAID.uiField2 = 1;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;

           eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
           if(eError != IEC61850_ERROR_NONE)
           {
               printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
           }

           //verticalSliderXcbr->setValue(2);
           bXCBRSwFlg = TRUE;
           bErthSwFlg = FALSE;
        }
        else
        {
            bValue = IEC61850_DB_POS_FALSE;
            printf("bValue: %d\n", bValue);

            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 4;
            tDAID.uiField2 = 1;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;

            eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
            if(eError != IEC61850_ERROR_NONE)
            {
                printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
            }
            //verticalSliderXcbr->setValue(1);
             bXCBRSwFlg = FALSE;
             bErthSwFlg = FALSE;
        }
    }
    else
    {
        if(cValue == TRUE)
        {
            bValue = IEC61850_DB_POS_TRUE;
            printf("bValue: %d\n", bValue);

            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 5;
            tDAID.uiField2 = 1;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;

           eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
           if(eError != IEC61850_ERROR_NONE)
           {
               printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
           }

           //verticalSliderCsw->setValue(2);
           bXCBRSwFlg = FALSE;
           bErthSwFlg = TRUE;
        }
        else
        {
            bValue = IEC61850_DB_POS_FALSE;
            printf("bValue: %d\n", bValue);

            tDAID.Generic_type = IEC61850_DAID_GENERIC;
            tDAID.uiField1 = 5;
            tDAID.uiField2 = 1;
            tDAID.uiField3 = 2;
            tDAID.uiField4 = 0;
            tDAID.uiField5 = 0;

            eError = IEC61850_Update(PIS10Server::pThis->GetMyServer(), (struct IEC61850_DataAttributeID *) &tDAID, &tDAData, 1);
            if(eError != IEC61850_ERROR_NONE)
            {
                printf("tDAID: %d %d %d %d %d,   %s (%i)\n", tDAID.uiField1, tDAID.uiField2, tDAID.uiField3, tDAID.uiField4, tDAID.uiField5, IEC61850_ErrorString(eError), eError);
            }
            //verticalSliderCsw->setValue(1);
            bXCBRSwFlg = FALSE;
            bErthSwFlg = FALSE;
        }
    }
}

/*****************************************************************************
* UpdateStateTimerHandler() Function
******************************************************************************
* Timer function handler, is called when timer fires. This functions checks the
* Global status flags and updates  QServerwidget GUI
******************************************************************************/
void QtServerWidget::UpdateStateTimerHandler()
{
    QPixmap pixmapESWOpenXCBROpen("switch00.png");
    QPixmap pixmapESWOpenXCBRClose("switch01.png");
    QPixmap pixmapESWCloseXCBROpen("switch10.png");
    QPixmap pixmapESWCloseXCBRClose("switch11.png");

    if(bLed1 == TRUE)
    {
        Led_Lbl1->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl1->setPixmap(*pixmapRedLed);
    }

    if(bLed2 == TRUE)
    {
        Led_Lbl2->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl2->setPixmap(*pixmapRedLed);
    }

    if(bLed3 == TRUE)
    {
        Led_Lbl3->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl3->setPixmap(*pixmapRedLed);
    }

    if(bLed4 == TRUE)
    {
        Led_Lbl4->setPixmap(*pixmapGreenLed);
    }
    else
    {
        Led_Lbl4->setPixmap(*pixmapRedLed);
    }

    //printf("bXCBRSwFlg: %d, bErthSwFlg: %d \n",bXCBRSwFlg,bErthSwFlg);

    if(bXCBRSwFlg == TRUE && bErthSwFlg == FALSE)
    {
       scene->addPixmap(pixmapESWOpenXCBRClose);
       verticalSliderXcbr->setValue(2);
       verticalSliderCsw->setValue(1);
    }
    else if(bXCBRSwFlg == FALSE && bErthSwFlg == TRUE)
    {
       scene->addPixmap(pixmapESWCloseXCBROpen);
       verticalSliderXcbr->setValue(1);
       verticalSliderCsw->setValue(2);
    }
    else if(bXCBRSwFlg == FALSE && bErthSwFlg == FALSE)
    {
       scene->addPixmap(pixmapESWOpenXCBROpen);
       verticalSliderXcbr->setValue(1);
       verticalSliderCsw->setValue(1);
    }


    if(iAlarmFlg)
    {
        timerAlarmColor->start(400);
        //printf("iAlarmFlg: %d\n",iAlarmFlg);
    }
    else
    {
        timerAlarmColor->stop();
        Led_LblAlrm->setPixmap(*pixmapGreyLed);
        //printf("iAlarmFlg: stop\n");
    }

}

/*****************************************************************************
* pshbtnCloseHandler() Function
******************************************************************************
* This fucntion is handler for Close/Exit button. This function is called when
* Close/Exit button is clicked, it stops PIS10 stack and frees it.
******************************************************************************/
void QtServerWidget ::pshbtnCloseHandler()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    //Stop PIS10 stack
    eErrorCode = IEC61850_Stop(PIS10Server::pThis->GetMyServer()); // Starts myServer

    if(eErrorCode != IEC61850_ERROR_NONE)
    {
        printf("   Can't start server: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    //Free PIS10 stack
    eErrorCode = IEC61850_Free(PIS10Server::pThis->GetMyServer());  //Frees myServerClient

    if(eErrorCode == IEC61850_ERROR_NONE)
    {
        //MyServer = NULL;
    }
    else
    {
        printf("   Free IEC61850 Object has failed: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

     qApp->quit();
}


/*****************************************************************************
* AlarmBtnColorChange() Function
******************************************************************************
* This fucntion is handler for Alarm timer.
* If Alarm is set then it flashes the led.
******************************************************************************/
void QtServerWidget :: AlarmBtnColorChange()
{
    //printf("iToggle: %d\n",iToggle);
    if(iToggle == 0)
    {
        Led_LblAlrm->setPixmap(*pixmapOrangeLed);
        iToggle = 1;
    }
    else
    {
        Led_LblAlrm->setPixmap(*pixmapGreyLed);
        iToggle = 0;
    }

}

/*****************************************************************************
* getRGBhexColor() Function
******************************************************************************
* This fucntion is getRGB color hex codes.
*
******************************************************************************/
QString QtServerWidget::getRGBhexColor(const QColor color)
{
    QString col("#");
    // qDebug("red: %x", color.red());
    if(color.red() == 0)
    col.append("00");
    else
    col.append(QString::number(color.red(), 16));
    // qDebug("green: %x", color.green());
    if(color.green() == 0)
    col.append("00");
    else
    col.append(QString::number(color.green(), 16));
    // qDebug("blue: %x", color.blue());
    if(color.blue() == 0)
    col.append("00");
    else
    col.append(QString::number(color.blue(), 16));
    return col;
}
