/*****************************************************************************/
/*!	\file 		QtServerWidget.h
 *	\brief          QtServer GUI header
 *	\par 		SystemCORP Embedded Technology Pty Ltd
 *
 *			5/50 William St.,
 *			Beckenham,
 *			Perth, WA 6107,
 *			Australia.
 *
 *			Phone	: +61 (8) 9258 4683
 *			Fax	: +61 (8) 9258 4673
 *			Email	: info@systemcorp.com.au
 */
/*****************************************************************************/
#ifndef QTSERVERWIDGET_H
#define QTSERVERWIDGET_H
#include <QtGui>
#include <QWidget>
#include <QtGui/QApplication>
#include <QGraphicsView>
#include <QGraphicsPixmapItem>
#include <QPixmap>
#include <QLabel>
#include <QTimeLine>
#include <qpixmap.h>

#include "ui_QtServerWidget.h"

#include "IEC61850Types.h"
#include "IEC61850API.h"
#include "sysctype.h"


QT_BEGIN_NAMESPACE
class QLabel;
class QPixmap;
class QPropertyAnimation;
QT_END_NAMESPACE

namespace Ui {
    class QtServerWidget;
}

class QtServerWidget : public QWidget, private Ui::QtServerWidgetDLG
{
    Q_OBJECT

public:
    explicit QtServerWidget(QWidget *parent = 0);
    ~QtServerWidget();
    void *ptrToPIS10;

private:
   // Ui::QtServerWidget *ui;
    QLCDNumber *LcdNum;

public slots:
    void pshbtnCloseHandler();
    void MaxAlarmHandler();
    void UpdateStateTimerHandler();
    void pshbtnUpdateAnIn1Handler();
    void pshbtnUpdateAnIn2Handler();
    void LedController(unsigned int uiLedIndx, Boolean bValue);
    void XcbrCswController(unsigned int uiIndx, unsigned char cValue);
    void pbDipSw1Handler();
    void pbDipSw2Handler();
    void pbDipSw3Handler();
    void pbDipSw4Handler();
    void AlarmBtnColorChange();
    QString getRGBhexColor(const QColor color);
public:
    QGraphicsScene *scene;
    QTimeLine *timeLine;
    QPixmap *pixmapGreenLed;
    QPixmap *pixmapRedLed;
    QPixmap *pixmapGreyLed;
    QPixmap *pixmapOrangeLed;
    QIcon *IconGreenLed;
    QIcon *IconRedLed;
    QTimer *timerUpdateState;
    QTimer *timerAlarmColor;
};

#endif // QTSERVERWIDGET_H
