// Data Types for application 
/*****************************************************************************/
/*!	\file			TestAppTypes.h 
 *	\brief 		Header file contains all the custom data types and #defines for the application
 *	\par 			SystemCORP Pty Ltd
 *
 *				15/50 William St.,
 *				Beckenham,
 *				Perth, WA 6107,
 *				Australia.
 *
 *				Phone	: +61 (8) 9258 4683
 *				Fax	: +61 (8) 9258 4673
 *				Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

/******************************************************************************
*	Defines
******************************************************************************/


#ifndef TEST_APP_TYPES_INCLUDED
#define TEST_APP_TYPES_INCLUDED 1

/*======= IMPORTANT ======= *
  * If you add an enum make sure you 
  * Update CmdTypeToString in

  * UtilityFunctions.c
  *======================*/
  
enum CommandType {
	CMD_READ	  = 0,
	CMD_WRITE	  = 1,
	CMD_SELECT	  = 2,
	CMD_OPERATE	  = 3,
	CMD_CANCEL	  = 4,
	CMD_UPDATE	  = 5,
	CMD_RESPONSE  = 6,
	CMD_UPDATE_ERROR_ARRAY = 7,
	CMD_IEC61850_OBJECT_MANAGMENT = 8,
	CMD_PING	  =9,
	CMD_SCL_TRANSFER = 10,
	CMD_CB_RESPONSE = 11, //CallBack Response
	CMD_SET_SCL_FILENAME = 12,
	FINAL_CMD_ENUM, // This must ALWAYS be the last enum in the list as it is used as the max number of enums
};


/*======= IMPORTANT ======= *
  * If you add an enum make sure you 
  * Update ApiFunctionTypeToString in 

  * UtilityFunctions.c
  *======================*/

enum APIFunctionType {
	API_CREATE	  = 0,
	API_LOAD	  = 1,
	API_START	  = 2,
	API_STOP	  = 3,
	API_FREE	  = 4,
	FINAL_API_ENUM, // This must ALWAYS be the last enum in the list as it is used as the max number of enums
};

enum CommandStatusCodes{	
	CMD_SUCCESS = 1,
	CMD_WAITING = 0,
	CMD_FAIL	= -1,
};

enum ApplicationType{	
	APP_SERVER	= 0,
	APP_CLIENT	= 1,
};

enum TestErrorCodes{
	TEST_APP_NO_ERROR 		 	=  0,
	TEST_APP_TCP_SEND_FAILED 	= -1,
	TEST_APP_INVALID_PARAMETERS = -2,
	TEST_APP_SOCKET_BLOCKED 	= -3,
	TEST_APP_ENCODING_FAILED 	= -4,
	TEST_APP_DECODING_FAILED	= -5,
	TEST_APP_FILE_FAILED_TO_LOAD= -6,
	TEST_APP_INVALID_SCL_FILE	= -7,
	TEST_APP_DAID_NOT_FOUND		= -8,
	TEST_APP_CMDID_NOT_FOUND	= -9,
	TEST_APP_ERROR_CODE_NOT_FOUND= -10,
	TEST_APP_INDEX_OUT_OF_BOUNDS = -11,
	TEST_APP_MALLOC_FAILED		= -12,
	TEST_APP_CHAR_NOT_FOUND		= -13,
	TEST_APP_CMD_TYPE_NOT_FOUND	= -14,
	TEST_APP_THREAD_FAILED_TO_START = -15,
	TEST_APP_GUI_UPDATE_FAILED	= -16,
	TEST_APP_POPUP_CANCELLED		= -17,
};



#endif
